unit VNC1L;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, SUISkinEngine;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    ComboBox1: TComboBox;
    OpenButton: TButton;
    ProgramButton: TButton;
    Label1: TLabel;
    SelectedFile: TLabel;
    Label2: TLabel;
    ProgressBar1: TProgressBar;
    procedure ComboBox1DropDown(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure ProgramButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


Type FT_Result = Integer;


// Device Info Node structure for info list functions
type FT_Device_Info_Node = record
  Flags         : DWord;
  DeviceType    : Dword;
  ID            : DWord;
  LocID         : DWord;
  SerialNumber  : array [0..15] of Char;
  Description   : array [0..63] of Char;
  DeviceHandle  : DWord;
end;


type TDWordptr = ^DWord;

// Structure to hold EEPROM data for FT_EE_Program function
TFT_Program_Data = record
  Signature1        : DWord;
  Signature2        : DWord;
  Version           : DWord;
  VendorID          : Word;
  ProductID         : Word;
  Manufacturer      : PChar;
  ManufacturerID    : PChar;
  Description       : PChar;
  SerialNumber      : PChar;
  MaxPower          : Word;
  PnP               : Word;
  SelfPowered       : Word;
  RemoteWakeup      : Word;
// Rev4 extensions
  Rev4              : Byte;
  IsoIn             : Byte;
  IsoOut            : Byte;
  PullDownEnable    : Byte;
  SerNumEnable      : Byte;
  USBVersionEnable  : Byte;
  USBVersion        : Word;
// FT2232C extensions
  Rev5              : Byte;
  IsoInA            : Byte;
  IsoInB            : Byte;
  IsoOutA           : Byte;
  IsoOutB           : Byte;
  PullDownEnable5   : Byte;
  SerNumEnable5     : Byte;
  USBVersionEnable5 : Byte;
  USBVersion5       : Word;
  AIsHighCurrent    : Byte;
  BIsHighCurrent    : Byte;
  IFAIsFifo         : Byte;
  IFAIsFifoTar      : Byte;
  IFAIsFastSer      : Byte;
  AIsVCP            : Byte;
  IFBIsFifo         : Byte;
  IFBIsFifoTar      : Byte;
  IFBIsFastSer      : Byte;
  BIsVCP            : Byte;
// FT232R extensions
  UseExtOsc         : Byte;
  HighDriveIOs      : Byte;
  EndpointSize      : Byte;
  PullDownEnableR   : Byte;
  SerNumEnableR     : Byte;
  InvertTXD         : Byte;
  InvertRXD         : Byte;
  InvertRTS         : Byte;
  InvertCTS         : Byte;
  InvertDTR         : Byte;
  InvertDSR         : Byte;
  InvertDCD         : Byte;
  InvertRI          : Byte;
  Cbus0             : Byte;
  Cbus1             : Byte;
  Cbus2             : Byte;
  Cbus3             : Byte;
  Cbus4             : Byte;
  RIsVCP            : Byte;
end;

var
  Form1: TForm1;

    FT_Handle : DWord = 0;
    FT_Current_Baud : Dword;
    FT_Current_DataBits : Byte;
    FT_Current_StopBits : Byte;
    FT_Current_Parity : Byte;
    FT_Current_FlowControl : Word;
    FT_RTS_On : Boolean;
    FT_DTR_On : Boolean;
    FT_Event_On : Boolean;
    FT_Error_On : Boolean;
    FT_XON_Value : Byte = $11;
    FT_XOFF_Value : Byte = $13;
    FT_EVENT_Value : Byte = $0;
    FT_ERROR_Value : Byte = $0;
// Used by CFGUnit to flag a bad value
    FT_SetupError : Boolean;
// Used to Return the current Modem Status
    FT_Modem_Status : DWord;
//  Used to return the number of bytes pending
//  in the Rx Buffer Queue
    //FT_Q_Bytes : DWord;
    FT_TxQ_Bytes : DWord;
    FT_Event_Status : DWord;
//  Used to Enable / Disable the Error Report Dialog
    FT_Enable_Error_Report : Boolean = True;
//  Deposit for Get latency timer
    FT_LatencyRd : Byte;

    FT_DeviceInfoList : array of FT_Device_Info_Node;

    Manufacturer: array [0..63] of char;
    ManufacturerID: array [0..15] of char;
    Description:  array [0..63] of char;
    SerialNumber:  array [0..15] of char;
    LocID : DWord;
    EEDataBuffer : TFT_Program_Data;
    UserData :  array [0..63] of byte;
    FT_UA_Size : integer;
    WordRead : Word;

    WriteBlockSize : integer = 128;
    ReadBlockSize : integer = 128;


Const
// FT_Result Values
    FT_OK = 0;
    FT_INVALID_HANDLE = 1;
    FT_DEVICE_NOT_FOUND = 2;
    FT_DEVICE_NOT_OPENED = 3;
    FT_IO_ERROR = 4;
    FT_INSUFFICIENT_RESOURCES = 5;
    FT_INVALID_PARAMETER = 6;
    FT_SUCCESS = FT_OK;
    FT_INVALID_BAUD_RATE = 7;
    FT_DEVICE_NOT_OPENED_FOR_ERASE = 8;
    FT_DEVICE_NOT_OPENED_FOR_WRITE = 9;
    FT_FAILED_TO_WRITE_DEVICE = 10;
    FT_EEPROM_READ_FAILED = 11;
    FT_EEPROM_WRITE_FAILED = 12;
    FT_EEPROM_ERASE_FAILED = 13;
    FT_EEPROM_NOT_PRESENT = 14;
    FT_EEPROM_NOT_PROGRAMMED = 15;
    FT_INVALID_ARGS = 16;
    FT_NOT_SUPPORTED = 17;
    FT_OTHER_ERROR = 18;
    FT_DEVICE_LIST_NOT_READY = 19;
// FT_Open_Ex Flags
    FT_OPEN_BY_SERIAL_NUMBER = 1;
    FT_OPEN_BY_DESCRIPTION = 2;
    FT_OPEN_BY_LOCATION = 4;
// FT_List_Devices Flags
    FT_LIST_NUMBER_ONLY = $80000000;
    FT_LIST_BY_INDEX = $40000000;
    FT_LIST_ALL = $20000000;
// Baud Rate Selection
    FT_BAUD_300 = 300;
    FT_BAUD_600 = 600;
    FT_BAUD_1200 = 1200;
    FT_BAUD_2400 = 2400;
    FT_BAUD_4800 = 4800;
    FT_BAUD_9600 = 9600;
    FT_BAUD_14400 = 14400;
    FT_BAUD_19200 = 19200;
    FT_BAUD_38400 = 38400;
    FT_BAUD_57600 = 57600;
    FT_BAUD_115200 = 115200;
    FT_BAUD_230400 = 230400;
    FT_BAUD_460800 = 460800;
    FT_BAUD_921600 = 921600;
// Data Bits Selection
    FT_DATA_BITS_7 = 7;
    FT_DATA_BITS_8 = 8;
// Stop Bits Selection
    FT_STOP_BITS_1 = 0;
    FT_STOP_BITS_2 = 2;
// Parity Selection
    FT_PARITY_NONE = 0;
    FT_PARITY_ODD = 1;
    FT_PARITY_EVEN = 2;
    FT_PARITY_MARK = 3;
    FT_PARITY_SPACE = 4;
// Flow Control Selection
    FT_FLOW_NONE = $0000;
    FT_FLOW_RTS_CTS = $0100;
    FT_FLOW_DTR_DSR = $0200;
    FT_FLOW_XON_XOFF = $0400;
// Purge Commands
    FT_PURGE_RX = 1;
    FT_PURGE_TX = 2;
// Notification Events
    FT_EVENT_RXCHAR = 1;
    FT_EVENT_MODEM_STATUS = 2;
// Modem Status
    CTS = $10;
    DSR = $20;
    RI = $40;
    DCD = $80;
// FT_Device_Types
    FT232BM = 0;
    FT8U232AM = 1;
    FT8U100AX = 2;
    UNKNOWN = 3;
    FT2232C = 4;
    FT232R = 5;





// IO Buffer Sizes
    FT_In_Buffer_Size = $10000;    // 64k
    FT_In_Buffer_Index = FT_In_Buffer_Size - 1;
    FT_Out_Buffer_Size = $10000;    // 64k
    FT_Out_Buffer_Index = FT_Out_Buffer_Size - 1;
// DLL Name
    FT_DLL_Name = 'FTD2XX.DLL';

var
// Declare Input and Output Buffers
   FT_In_Buffer : Array[0..FT_In_Buffer_Index] of Byte;
   FT_Out_Buffer : Array[0..FT_Out_Buffer_Index] of Byte;
// A variable used to detect time-outs
// Attach a timer to the main project form
// which decrements this every 10mS if
// FT_TimeOut_Count <> 0
   FT_TimeOut_Count : Integer = 0;
// Used to determine how many bytes were
// actually received by FT_Read_Device_All
// in the case of a time-out
   FT_All_Bytes_Received : Integer = 0;
   FT_IO_Status : Ft_Result = FT_OK;
// Used By FT_ListDevices
   FT_Device_Count : DWord;
   FT_Device_String_Buffer : array [1..50] of Char;
   FT_Device_String : String;
   FT_Device_Location : DWord;
   USB_Device_Info_Node : FT_Device_Info_Node;
   FT_Event_Handle : DWord;
   OutIndex : integer;


implementation
{$R *.dfm}
{$R vista.RES}  // Use XP themes and Vista run level


//Classic functions
function FT_GetNumDevices(pvArg1:Pointer; pvArg2:Pointer; dwFlags:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_ListDevices';
function FT_ListDevices(pvArg1:Dword; pvArg2:Pointer; dwFlags:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_ListDevices';
function FT_Open(Index:Integer; ftHandle:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_Open';
function FT_OpenEx(pvArg1:Pointer; dwFlags:Dword; ftHandle:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_OpenEx';
function FT_OpenByLocation(pvArg1:DWord; dwFlags:Dword; ftHandle:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_OpenEx';
function FT_Close(ftHandle:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_Close';
function FT_Read(ftHandle:Dword; ReadBuf:Pointer; NumBytesToRead:DWord; NumBytesRead:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_Read';
function FT_Write(ftHandle:Dword; WriteBuf:Pointer; NumBytesToWrite:DWord; NumBytesWritten:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_Write';
function FT_ResetDevice(ftHandle:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_ResetDevice';
function FT_SetBaudRate(ftHandle:Dword; BaudRate:DWord):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetBaudRate';
function FT_SetDivisor(ftHandle:Dword; Divisor:DWord):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetDivisor';
function FT_SetDataCharacteristics(ftHandle:Dword; WordLength,StopBits,Parity:Byte):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetDataCharacteristics';
function FT_SetFlowControl(ftHandle:Dword; FlowControl:Word; XonChar,XoffChar:Byte):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetFlowControl';
function FT_SetDtr(ftHandle:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetDtr';
function FT_ClrDtr(ftHandle:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_ClrDtr';
function FT_SetRts(ftHandle:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetRts';
function FT_ClrRts(ftHandle:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_ClrRts';
function FT_GetModemStatus(ftHandle:Dword; ModemStatus:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetModemStatus';
function FT_SetChars(ftHandle:Dword; EventChar,EventCharEnabled,ErrorChar,ErrorCharEnabled:Byte):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetChars';
function FT_Purge(ftHandle:Dword; Mask:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_Purge';
function FT_SetTimeouts(ftHandle:Dword; ReadTimeout,WriteTimeout:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetTimeouts';
function FT_GetQueueStatus(ftHandle:Dword; RxBytes:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetQueueStatus';
function FT_SetBreakOn(ftHandle:Dword) : FT_Result; stdcall; External FT_DLL_Name name 'FT_SetBreakOn';
function FT_SetBreakOff(ftHandle:Dword) : FT_Result; stdcall; External FT_DLL_Name name 'FT_SetBreakOff';
function FT_GetStatus(ftHandle:DWord; RxBytes,TxBytes,EventStatus:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetStatus';
function FT_SetEventNotification(ftHandle:DWord; EventMask:DWord; pvArgs:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetEventNotification';
function FT_GetDeviceInfo(ftHandle:DWord; DevType,ID,SerNum,Desc,pvDummy:Pointer) : FT_Result; stdcall; External FT_DLL_Name name 'FT_GetDeviceInfo';
function FT_SetResetPipeRetryCount(ftHandle:Dword; RetryCount:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetResetPipeRetryCount';
function FT_StopInTask(ftHandle:Dword) : FT_Result; stdcall; External FT_DLL_Name name 'FT_StopInTask';
function FT_RestartInTask(ftHandle:Dword) : FT_Result; stdcall; External FT_DLL_Name name 'FT_RestartInTask';
function FT_ResetPort(ftHandle:Dword) : FT_Result; stdcall; External FT_DLL_Name name 'FT_ResetPort';
function FT_CyclePort(ftHandle:Dword) : FT_Result; stdcall; External 'FTD2XX.DLL' name 'FT_CyclePort';
function FT_CreateDeviceInfoList(NumDevs:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_CreateDeviceInfoList';
function FT_GetDeviceInfoList(pFT_Device_Info_List:Pointer; NumDevs:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetDeviceInfoList';
function FT_GetDeviceInfoDetail(Index:DWord; Flags,DevType,ID,LocID,SerialNumber,Description,DevHandle:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetDeviceInfoDetail';
function FT_GetDriverVersion(ftHandle:Dword; DrVersion:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetDriverVersion';
function FT_GetLibraryVersion(LbVersion:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetLibraryVersion';

// EEPROM functions
function FT_EE_Read(ftHandle:DWord; pEEData:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_EE_Read';
function FT_EE_Program(ftHandle:DWord; pEEData:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_EE_Program';
// EEPROM primitives - you need an NDA for EEPROM checksum
function FT_ReadEE(ftHandle:DWord; WordAddr:DWord; WordRead:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_ReadEE';
function FT_WriteEE(ftHandle:DWord; WordAddr:DWord; WordData:word):FT_Result; stdcall; External FT_DLL_Name name 'FT_WriteEE';
function FT_EraseEE(ftHandle:DWord):FT_Result; stdcall; External FT_DLL_Name name 'FT_EraseEE';
function FT_EE_UARead(ftHandle:DWord; Data:Pointer; DataLen:DWord; BytesRead:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_EE_UARead';
function FT_EE_UAWrite(ftHandle:DWord; Data:Pointer; DataLen:DWord):FT_Result; stdcall; External FT_DLL_Name name 'FT_EE_UAWrite';
function FT_EE_UASize(ftHandle:DWord; UASize:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_EE_UASize';

// FT2232C, FT232BM and FT245BM Extended API Functions
function FT_GetLatencyTimer(ftHandle:Dword; Latency:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetLatencyTimer';
function FT_SetLatencyTimer(ftHandle:Dword; Latency:Byte):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetLatencyTimer';
function FT_GetBitMode(ftHandle:Dword; BitMode:Pointer):FT_Result; stdcall; External FT_DLL_Name name 'FT_GetBitMode';
function FT_SetBitMode(ftHandle:Dword; Mask,Enable:Byte):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetBitMode';
function FT_SetUSBParameters(ftHandle:Dword; InSize,OutSize:Dword):FT_Result; stdcall; External FT_DLL_Name name 'FT_SetUSBParameters';






function SendBytes(DeviceHandle:integer; NumBytesToWrite:integer; var BytesWritten:integer):integer;
var
  Res : integer;
begin
  FT_Write(DeviceHandle,@FT_Out_Buffer,NumBytesToWrite,@BytesWritten);
  OutIndex := OutIndex - BytesWritten;
  if NumBytesToWrite = BytesWritten then
    Res := FT_OK
  else
    Res := FT_FAILED_TO_WRITE_DEVICE;
  Result := Res;
end;




procedure AddToBuffer(ByteToWrite:byte);
begin
  FT_Out_Buffer[OutIndex]:= ByteToWrite;
  OutIndex := OutIndex + 1;
end;





procedure TForm1.ComboBox1DropDown(Sender: TObject);
var
  i : integer;
  Count : integer;
  Res : FT_Result;
  DeviceInfoList : array of FT_Device_Info_Node;
begin
  // Reset device count
  Count := 0;
  ComboBox1.Items.Clear;
  Combobox1.Text := '';
  //form1.Refresh;
  Application.ProcessMessages;
  Res := FT_DEVICE_LIST_NOT_READY;

  // Get number of devices
  while (Res = FT_DEVICE_LIST_NOT_READY) do
    begin
      Res := FT_CreateDeviceInfoList(@Count);
      //if Res <> 0 then MessageDlg('Status : ' + IntToStr(Res), mtInformation,[mbOK],0);
    end;

  // Check status
  if Res = FT_OK then
    begin
    // Check that at least one device connected
      if Count = 0 then
        begin
          MessageDlg('No devices found.',mtInformation,[mbOK],0);
          Abort;
        end;
    end;

  // Size info list array to correct number of devices
  SetLength(DeviceInfoList, Count);

  // Populate list
  FT_GetDeviceInfoList(DeviceInfoList,@Count);

  // Populate combo box with descriptions of devices found
  for i := 0 to (Count-1) do
    begin
      ComboBox1.Items.Add(DeviceInfoList[i].Description);
    end;
end;





function GetAck(DeviceHandle:integer):boolean;
var
  Res : integer;
  NumBytesRead : DWord;
  FT_Q_Bytes : DWord;
  TimeoutValue : integer;
  Timeout : Boolean;
  StartTime : double;
  AckTime : double;
  Hour : word;
  Min : word;
  Sec : word;
  MSec : word;
begin
  GetAck := FALSE;
  NumBytesRead := 0;
  TimeoutValue := 2;    // 2 second timeout
  Timeout := FALSE;
  StartTime := Time;
  repeat
    Res := FT_GetQueueStatus(DeviceHandle,@FT_Q_Bytes);
    if FT_Q_Bytes > 0 then
      begin
        Res := FT_Read(DeviceHandle,@FT_In_Buffer,1,@NumBytesRead);
        if FT_In_Buffer[NumBytesRead-1] = $02 then
          GetAck := TRUE;
      end
    else
      begin
        sleep(0);
      end;
    Application.ProcessMessages;
    AckTime := Time - StartTime;
    DecodeTime(AckTime,Hour,Min,Sec,MSec);
    if Sec > TimeoutValue then
      Timeout := TRUE;
  until (NumBytesRead > 0) OR Timeout;
  if Timeout then
    begin
      GetAck := FALSE;
      //MessageDlg('Failed to read ACK.',mtInformation,[mbOK],0);
    end;
end;





function SyncToVNC1L(DeviceHandle:integer):boolean;
var
  Synchronised : Boolean;
  BytesWritten : integer;
  NumBytesRead : integer;
  TimeoutValue : integer;
  Timeout : Boolean;
  StartTime : double;
  SyncTime : double;
  Hour : word;
  Min : word;
  Sec : word;
  MSec : word;
  i : integer;
  Res : integer;
  FT_Q_Bytes : DWord;
begin
  NumBytesRead := 0;
  TimeoutValue := 2;    // 2 second timeout
  Timeout := FALSE;
  StartTime := Time;
  Synchronised := FALSE;
  repeat
    AddToBuffer($FF);     // This command is echoed by the bootloader
    BytesWritten := 0;
    SendBytes(DeviceHandle,1,BytesWritten);
    Res := FT_GetQueueStatus(DeviceHandle,@FT_Q_Bytes);
    if FT_Q_Bytes > 0 then
      begin
        Res := FT_Read(Devicehandle,@FT_In_Buffer,FT_Q_Bytes,@NumBytesRead);
      end
    else
      begin
        sleep(0);
      end;
    Application.ProcessMessages;
    SyncTime := Time - StartTime;
    DecodeTime(SyncTime,Hour,Min,Sec,MSec);
  until (FT_In_Buffer[NumBytesRead-1] = $FF) OR (Sec > TimeoutValue);

  AddToBuffer($FA);       // This command is also echoed by the bootloader
  BytesWritten := 0;
  SendBytes(DeviceHandle,1,BytesWritten);
  StartTime := Time;      // Restart timeout
  repeat
    Res := FT_GetQueueStatus(DeviceHandle,@FT_Q_Bytes);
    if FT_Q_Bytes > 0 then
      begin
        Res := FT_Read(Devicehandle,@FT_In_Buffer,FT_Q_Bytes,@NumBytesRead);
      end
    else
      begin
        sleep(0);
      end;
    Application.ProcessMessages;
    SyncTime := Time - StartTime;
    DecodeTime(SyncTime,Hour,Min,Sec,MSec);
    Timeout := (Sec > TimeoutValue);
  until (FT_In_Buffer[NumBytesRead-1] = $FA) OR (Sec > TimeoutValue);

  // if the last byte received was $FA, then synchronised
  if (Timeout) then
    Synchronised := FALSE
  else if (FT_In_Buffer[NumBytesRead-1] = $FA) AND NOT Timeout then
    Synchronised := TRUE;

  Result := Synchronised;
end;




function SetupVNC1L(DeviceHandle:dword;BaudRate:integer):integer;
var
  Res : integer;
  BytesWritten : integer;
begin
  // Reset out count
  //OutIndex := 0;
  // Set Baud rate to 1MBaud - Talking to bootloader, not firmware
  AddToBuffer($01);                   // SetVars
  AddToBuffer(6000 AND $FF);          // TimerLow
  AddToBuffer((6000 shr 8) AND $FF);  // TimerHigh
  if BaudRate = 115200 then
    begin
      AddToBuffer($1A);                   // Baud1
      AddToBuffer($00);                   // Baud2
      AddToBuffer($00);                   // Baud3
    end
  else if BaudRate = 1000000 then
    begin
      AddToBuffer($03);                   // Baud1
      AddToBuffer($00);                   // Baud2
      AddToBuffer($00);                   // Baud3
    end
  else if BaudRate = 3000000 then
    begin
      AddToBuffer($03);                   // Baud1
      AddToBuffer($00);                   // Baud2
      AddToBuffer($00);                   // Baud3
    end;
  Res := SendBytes(DeviceHandle,OutIndex,BytesWritten);
  Sleep(100);
  //GetAck(DeviceHandle);
  Result := Res;
end;





procedure SetRunMode(DeviceHandle:integer);
  // CBus2 = PROGn
  // CBus3 = RESETn
const
  ResetOff_ProgOff = $CC;
  ResetOff_ProgOn = $C8;
  ResetOn_ProgOff = $C4;
  ResetOn_ProgOn = $C0;
var
  Res: integer;
begin
  Res := FT_SetBitMode(DeviceHandle,ResetOn_ProgOff,$20);
  sleep(10);
  Res := FT_SetBitMode(DeviceHandle,ResetOff_ProgOff,$20);
  sleep(10);
end;





procedure SetProgramMode(DeviceHandle:integer);
  // CBus2 = PROGn
  // CBus3 = RESETn
const
  ResetOff_ProgOff = $CC;
  ResetOff_ProgOn = $C8;
  ResetOn_ProgOff = $C4;
  ResetOn_ProgOn = $C0;
var
  Res: integer;
begin
  Res := FT_SetBitMode(DeviceHandle,ResetOn_ProgOn,$20);
  sleep(10);
  Res := FT_SetBitMode(DeviceHandle,ResetOff_ProgOn,$20);
  sleep(10);
end;





function SetupUART(DeviceHandle:Dword; BaudRate:integer):integer;
var
  Res : integer;
begin
  // Set Baud rate to 1MBaud
  Res := FT_SetBaudRate(DeviceHandle,BaudRate);
  if Res = FT_OK then
  // Set 8 data bits, 1 stop bit, no parity
    Res := FT_SetDataCharacteristics(DeviceHandle,FT_DATA_BITS_8,FT_STOP_BITS_1,FT_PARITY_NONE);
  if Res = FT_OK then
  // Set RTS/CTS handshaking
    Res := FT_SetFlowControl(DeviceHandle,FT_FLOW_RTS_CTS,0,0);
  Result := Res;
  if Res = FT_OK then
    Res := FT_SetTimeouts(DeviceHandle,1000,1000);
end;




function InitUART(DeviceHandle:Dword):integer;
begin
  FT_SetRTS(DeviceHandle);
  FT_SetDTR(DeviceHandle);
  Result := SetupUART(DeviceHandle,115200);
end;





procedure WriteFlashAddress(DeviceHandle:integer; Address:integer);
var
  BytesWritten : integer;
begin
AddToBuffer($03);                     // Write Flash Block command
AddToBuffer(Address AND $FF);         // FlashAddressLow
AddToBuffer((Address SHR 8) AND $FF); // FlashAddressHigh
AddToBuffer(WriteBlockSize);          // Count
SendBytes(DeviceHandle,OutIndex,BytesWritten);
end;




procedure ReadFlashAddress(DeviceHandle:integer; Address:integer);
var
  BytesWritten : integer;
begin
AddToBuffer($02);                     // Read Flash Block command
AddToBuffer(Address AND $FF);         // FlashAddressLow
AddToBuffer((Address SHR 8) AND $FF); // FlashAddressHigh
AddToBuffer(ReadBlockSize);           // Count
SendBytes(DeviceHandle,OutIndex,BytesWritten);
end;




function ProgramFlashFromFile(DeviceHandle:integer; Filename:TFilename):Boolean;
var
  ROMFile : File;
  FlashAddress : integer;
  NumRead : integer;
  SectorBuf : Array[0..511] of byte;
  i : integer;
  BytesWritten : integer;
  FT_Q_Bytes : integer;
begin
  // Show progress bar
  form1.ProgressBar1.Visible := TRUE;
  // Set progress bar range and start point
  form1.ProgressBar1.Min := 0;
  form1.ProgressBar1.Max := 65536;
  form1.ProgressBar1.Position := 0;
  // Hide label
  form1.SelectedFile.Visible := FALSE;
  // Change label 2
  form1.Label2.Caption := 'Programming Flash:';
  // Disable controls
  form1.ComboBox1.Enabled := FALSE;
  form1.OpenButton.Enabled := FALSE;
  form1.ProgramButton.Enabled := FALSE;

  // Open file
  AssignFile(ROMFile,Filename);
  Reset(ROMFile,1);
  FlashAddress := 0;

  repeat
    BlockRead(ROMFile,SectorBuf,WriteBlockSize,NumRead);
    if NumRead > 0 then
      begin
        WriteFlashAddress(DeviceHandle,FlashAddress);
        //GetAck(DeviceHandle);
        if NOT GetAck(DeviceHandle) then
          begin
            MessageDlg('Failed to read ACK.',mtInformation,[mbOK],0);
            Result := FALSE;
            Exit;
          end;
        for i := 0 to (WriteBlockSize-1) do
          begin
            AddToBuffer(SectorBuf[i]);
          end;
        SendBytes(DeviceHandle,OutIndex,BytesWritten);
        //GetAck(DeviceHandle);
        if NOT GetAck(DeviceHandle) then
          begin
            MessageDlg('Failed to read ACK.',mtInformation,[mbOK],0);
            Result := FALSE;
            Exit;
          end;
        FlashAddress := FlashAddress + WriteBlockSize;
        // Update progress bar
        form1.ProgressBar1.Position := FlashAddress;
        //form1.Refresh;
        Application.ProcessMessages;
      end
    else
      begin
        //done := true;
      end;
    Application.ProcessMessages;
  until (NumRead = 0);
  Application.ProcessMessages;

  // Close file
  CloseFile(ROMFile);

  Result := TRUE;
end;



function VerifyFlash(DeviceHandle:integer; Filename:TFilename):Boolean;
var
  ROMFile : File;
  FlashAddress : integer;
  NumRead : integer;
  SectorBuf : Array[0..511] of byte;
  i : integer;
  BytesWritten : integer;
  FT_Q_Bytes : integer;
  FlashVerified : Boolean;
  Res : FT_Result;
  NumBytesRead : integer;
begin
  // Show progress bar
  form1.ProgressBar1.Visible := TRUE;
  // Set progress bar range and start point
  form1.ProgressBar1.Min := 0;
  form1.ProgressBar1.Max := 65536;
  form1.ProgressBar1.Position := 0;
  // Hide label
  form1.SelectedFile.Visible := FALSE;
  // Change label 2
  form1.Label2.Caption := 'Verifying Flash:';
  // Disable controls
  form1.ComboBox1.Enabled := FALSE;
  form1.OpenButton.Enabled := FALSE;
  form1.ProgramButton.Enabled := FALSE;

  // Adjust Baud rates
  //Res := SetUpVNC1L(DeviceHandle,3000000);
  //Res := SetupUART(DeviceHandle,3000000);
  //GetAck(DeviceHandle);
  // If doing this, consider re-syncing


  // Open file
  AssignFile(ROMFile,Filename);
  Reset(ROMFile,1);
  FlashAddress := 0;
  FlashVerified := TRUE;

////////////////////
  Res := FT_GetQueueStatus(DeviceHandle,@FT_Q_Bytes);
  if FT_Q_Bytes > 0 then
    Res := FT_Read(DeviceHandle,@FT_In_Buffer,FT_Q_Bytes,@NumBytesRead);
////////////////////

  repeat
    BlockRead(ROMFile,SectorBuf,ReadBlockSize,NumRead);
    if NumRead > 0 then
      begin
        ReadFlashAddress(DeviceHandle,FlashAddress);
        //GetAck(DeviceHandle);
        if NOT GetAck(DeviceHandle) then
          begin
            MessageDlg('Failed to read ACK.',mtInformation,[mbOK],0);
            Result := FALSE;
            Exit;
          end;
        FT_Q_Bytes := 0;
        repeat
          begin
            Res := FT_GetQueueStatus(DeviceHandle, @FT_Q_Bytes);
            Sleep(0);
          end;
        until (FT_Q_Bytes >= ReadBlockSize);  //OR timeout

        Res := FT_Read(DeviceHandle,@FT_In_Buffer,ReadBlockSize,@NumBytesRead);

        for i := 1 to ReadBlockSize do
          begin
            if SectorBuf[i-1] <> FT_In_Buffer[i-1] then
              begin
                FlashVerified := FALSE;
                MessageDlg('Failed Flash verification. (Address ' + IntToStr(FlashAddress + i) + ')',mtInformation,[mbOK],0);
                Result := FlashVerified;
                Exit;
              end;
          end;
        FlashAddress := FlashAddress + ReadBlockSize;
        // Update progress bar
        form1.ProgressBar1.Position := FlashAddress;
        //form1.Refresh;
        Application.ProcessMessages;
      end
    else
      begin
        //done := true;
      end;
    Application.ProcessMessages;
  until (NumRead = 0);
  // Close file
  CloseFile(ROMFile);
  MessageDlg('VNC1L-1A Flash reprogrammed and verified.',mtinformation,[mbOK],0);

  Result := FlashVerified;
end;




procedure TForm1.OpenButtonClick(Sender: TObject);
var
  filename : TFilename;
  CurrentDir : string;
begin
  GetDir(0,CurrentDir);
  OpenDialog1.InitialDir := CurrentDir;
  if OpenDialog1.Execute then
    begin
      filename := OpenDialog1.FileName;
      SelectedFile.Caption := filename;
    end;
end;





procedure TForm1.ProgramButtonClick(Sender: TObject);
var
  Res : FT_Result;
  BRes : Boolean;
  DeviceDescription : string;
  Sync : boolean;
  ROMFile : File;
  StartTime : double;
  SyncTime : double;
  Hour : word;
  Min : word;
  Sec : word;
  MSec : word;
  SyncTimeout : integer;
  FT_Q_Bytes : integer;
  NumBytesRead : integer;
begin
  // Disable controls
  form1.ComboBox1.Enabled := FALSE;
  form1.OpenButton.Enabled := FALSE;
  form1.ProgramButton.Enabled := FALSE;
  //form1.Refresh;
  Application.ProcessMessages;

  // Check that a device has been selected i.e. combobox is not empty
  if Combobox1.Text = '' then
    begin
      MessageDlg('A device must be selected to program the VNC1L-1A Flash.',mtInformation,[mbOK],0);
        // Enable controls
        form1.ComboBox1.Enabled := TRUE;
        form1.OpenButton.Enabled := TRUE;
        form1.ProgramButton.Enabled := TRUE;
        Application.ProcessMessages;
        Exit;
    end;

  // Check that a ROM filename has been selected
  if SelectedFile.Caption = '' then
    begin
      MessageDlg('A ROM file has not been selected.' + Chr(13) + Chr(10) + 'Please select a ROM file to program into the VNC1L.',mtINformation,[mbOK],0);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;

  // Check that selected file is 64k in size
  AssignFile(ROMFile,SelectedFile.Caption);
  Reset(ROMFile,1);
  if FileSize(ROMFile) <> 65536 then
    begin
      MessageDlg('The ROM file selected does not have a size of 64K.',mtInformation,[mbOK],0);
      CloseFile(ROMFile);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;
  CloseFile(ROMFile);

  // Open device
  Res := FT_OpenEx(PChar(ComboBox1.Text),FT_OPEN_BY_DESCRIPTION,@FT_Handle);
  if Res <> 0 then
    begin
      MessageDlg('Failed to open ' + DeviceDescription,mtInformation,[mbOK],0);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;

  // Set program mode
  SetProgramMode(FT_Handle);

  // Initialise UART - 115200, RTS/CTS handshaking, 8 data bits, 1 stop bit
  Res := InitUART(FT_Handle);
  if Res <> 0 then
    begin
      MessageDlg('Failed to initialise UART.',mtInformation,[mbOK],0);
      FT_Close(FT_Handle);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;

  // Check synchronisation            - This FAILS here!  Why?
  {Sync := SyncToVNC1L(FT_Handle);
  if NOT Sync then
    begin
      MessageDlg('Failed to synchronise with VNC1L-1A.' + Chr(13) + Chr(10) + 'Check programming jumper setting if available and reset or re-power the VNC1L.',mtInformation,[mbOK],0);
      FT_Close(FT_Handle);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;   }

  // Set up VNC1L UART - defaults to 115200, RTS/CTS handshaking, 8 data bits, 1 stop bit, no parity
  // Set it to 1MBaud, RTS/CTS handshaking, 8 data bits, 1 stop bit, no parity
  Res := SetupVNC1L(FT_Handle,1000000);
  if Res <> 0 then
    begin
      MessageDlg('Failed to initialise VNC1L.',mtInformation,[mbOK],0);
      FT_Close(FT_Handle);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;

  // Setup UART - 1000000 Baud, RTS/CTS handshaking, no parity, 8 data bits, 1 stop bit, no parity
  Res := SetupUART(FT_Handle,1000000);
  if Res <> 0 then
    begin
      MessageDlg('Failed to set up UART.',mtInformation,[mbOK],0);
      FT_Close(FT_Handle);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;

  // Check synchronisation
  Sync := SyncToVNC1L(FT_Handle);
  if NOT Sync then
    begin
      MessageDlg('Failed to synchronise with VNC1L-1A.' + Chr(13) + Chr(10) + 'Check programming jumper setting if available and reset or re-power the VNC1L.',mtInformation,[mbOK],0);
      FT_Close(FT_Handle);
      // Enable controls
      form1.ComboBox1.Enabled := TRUE;
      form1.OpenButton.Enabled := TRUE;
      form1.ProgramButton.Enabled := TRUE;
      Application.ProcessMessages;
      Exit;
    end;


  // Program Vinculum from file
  BRes := ProgramFlashFromFile(FT_Handle, SelectedFile.Caption);
  if BRes then
    // Verify flash after programming
    VerifyFlash(FT_Handle, SelectedFile.Caption);

  // Hide progress bar
  form1.ProgressBar1.Visible := FALSE;
  // Show selected filename
  form1.SelectedFile.Visible := TRUE;
  // Change label 2 caption back
  form1.Label2.Caption := 'Selected File:';
  // Enable controls
  form1.ComboBox1.Enabled := TRUE;
  form1.OpenButton.Enabled := TRUE;
  form1.ProgramButton.Enabled := TRUE;
  //form1.Refresh;
  Application.ProcessMessages;

  // Return VNC1L to 115200 Baud for devices that have jumpers
  SetupVNC1L(FT_Handle,115200);
  SetupUART(FT_Handle,115200);

  // Set run mode
  SetRunMode(FT_Handle);

  // Close device
  Res := FT_Close(FT_Handle);
  if Res <> 0 then
    MessageDlg('Failed to close ' + DeviceDescription,mtInformation,[mbOK],0);
end;




procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FT_Close(FT_Handle);
end;



end.
